import logging.config

logging.config.dictConfig(
    {
        "version": 1,
        "formatters": {"verbose": {"format": "%(name)s: %(message)s"}},
        "handlers": {
            "console": {
                "level": "DEBUG",
                "class": "logging.StreamHandler",
                "formatter": "verbose",
            },
        },
        "loggers": {
            "zeep.transports": {
                "level": "DEBUG",
                "propagate": True,
                "handlers": ["console"],
            },
        },
    }
)



import argparse
import sys

from connectsensor import SensorClient, APIError


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--username", required=True, help="Kingspan account email address"
    )
    parser.add_argument("--password", required=True, help="Kingspan account password")
    parser.add_argument("--debug", action="store_true")

    args = parser.parse_args()

    if args.debug:
        import connectsensor.debug

    client = SensorClient()
    try:
        client.login(args.username, args.password)
    except APIError as e:
        if "Authentication Failed" in str(e):
            print(
                "Authentication Failed: invalid username or password", file=sys.stderr
            )
        else:  # pragma: no cover
            print("Unknown API error:", e.value, file=sys.stderr)
        sys.exit(1)

    for tank in client.tanks:
        tank_level = tank.level
        tank_level_percent = str(int(100 * tank.level / tank.capacity))

        print(tank.name + ":")
        print(f"\tCapacity = {tank.capacity}")
        print(f"\tSerial Number = {tank.serial_number}")
        print(f"\tModel = {tank.model}")
        print(f"\tLevel = {tank_level_percent}% ({tank.level} litres)")
        print(f"\tLast Read = {tank.last_read}")

        print("\nHistory:")
        print("\t{0:<22} {1:<6} {2:<5}".format("Reading date", "%Full", "Litres"))
        for index, measurement in tank.history.iterrows():
            print(
                "\t{0:<22} {1:<6} {2:<5}".format(
                    measurement.reading_date.strftime("%d-%b-%Y %H:%M"),
                    measurement.level_percent,
                    measurement.level_litres,
                )
            )


if __name__ == "__main__":
    # execute only if run as a script
    main()



from async_property import async_property
from zeep.helpers import serialize_object

import pandas as pd


class Tank:
    def __init__(self, soap_client: object, signalman_no: str):
        self._soap_client = soap_client
        self._signalman_no = signalman_no
        self._level_data = None

    @property
    def level(self) -> int:
        """Return the oil level in the tank in litres"""
        self._cache_tank_data()
        return int(self._level_data["LevelLitres"])

    @property
    def serial_number(self) -> str:
        self._cache_tank_data()
        return self._tank_info["Serial No"]

    @property
    def model(self) -> str:
        self._cache_tank_data()
        return self._tank_info["Model"]

    @property
    def name(self) -> str:
        self._cache_tank_data()
        return self._tank_info["Tank Name"]

    @property
    def capacity(self) -> int:
        """Return the capacity of the tank in litres"""
        self._cache_tank_data()
        return int(self._tank_info["Tank Capacity(L)"])

    @property
    def last_read(self) -> str:
        """Return the last read date of the tank as a datetime object"""
        self._cache_tank_data()
        return self._level_data["ReadingDate"]

    @property
    def history(self):
        history_data = self._soap_client._get_history(self._signalman_no)
        return history_to_df(history_data)

    def _cache_tank_data(self):
        if self._level_data is None:
            response = self._soap_client._get_latest_level(self._signalman_no)
            unpack_tank_data(self, response)


class AsyncTank:
    def __init__(self, soap_client: object, signalman_no: str):
        self._soap_client = soap_client
        self._signalman_no = signalman_no
        self._level_data = None

    @async_property
    async def level(self) -> int:
        await self._cache_tank_data()
        return int(self._level_data["LevelLitres"])

    @async_property
    async def serial_number(self) -> str:
        await self._cache_tank_data()
        return self._tank_info["Serial No"]

    @async_property
    async def model(self) -> str:
        await self._cache_tank_data()
        return self._tank_info["Model"]

    @async_property
    async def name(self) -> str:
        await self._cache_tank_data()
        return self._tank_info["Tank Name"]

    @async_property
    async def capacity(self) -> int:
        await self._cache_tank_data()
        return int(self._tank_info["Tank Capacity(L)"])

    @async_property
    async def last_read(self) -> str:
        """Return the last read date of the tank as a datetime object"""
        await self._cache_tank_data()
        return self._level_data["ReadingDate"]

    @async_property
    async def history(self):
        history_data = await self._soap_client._get_history(self._signalman_no)
        return history_to_df(history_data)

    async def _cache_tank_data(self):
        if self._level_data is None:
            response = await self._soap_client._get_latest_level(self._signalman_no)
            unpack_tank_data(self, response)


def unpack_tank_data(cls, response):
    cls._level_data = serialize_object(response["Level"])
    tank_info = serialize_object(response["TankInfo"])
    cls._tank_info = {x["Name"]: x["Value"] for x in tank_info["APITankInfoItem"]}


def history_to_df(history_data):
    df = pd.DataFrame(serialize_object(history_data))
    df = df[["ReadingDate", "LevelPercentage", "LevelLitres"]]
    df.columns = ["reading_date", "level_percent", "level_litres"]
    return df



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['connectsensor']

package_data = \
{'': ['*']}

install_requires = \
['async-property>=0.2.1,<0.3.0',
 'asyncio>=3.4.3,<4.0.0',
 'datetime>=4.7,<5.0',
 'httpx>=0.23.0,<0.24.0',
 'pandas>=1.4.3,<2.0.0',
 'requests>=2.28.1,<3.0.0',
 'zeep>=4.2.0,<5.0.0']

entry_points = \
{'console_scripts': ['kingspan-export = connectsensor._kingspan_export:main',
                     'kingspan-notifier = '
                     'connectsensor._kingspan_notifier:main',
                     'kingspan-status = connectsensor._kingspan_status:main']}

setup_kwargs = {
    'name': 'kingspan-connect-sensor',
    'version': '2.1.0',
    'description': 'API to get oil tank from Kingspan SENSiT sensors',
    'long_description': '# kingspan-connect-sensor\n\n[![build:](https://github.com/masaccio/kingspan-connect-sensor/actions/workflows/run-all-tests.yml/badge.svg)](https://github.com/masaccio/kingspan-connect-sensor/actions/workflows/run-all-tests.yml)\n[![build:](https://github.com/masaccio/kingspan-connect-sensor/actions/workflows/codeql.yml/badge.svg)](https://github.com/masaccio/kingspan-connect-sensor/actions/workflows/codeql.yml)\n<!-- [![codecov](https://codecov.io/gh/masaccio/kingspan-connect-sensor/branch/main/graph/badge.svg?token=EKIUFGT05E)](https://codecov.io/gh/masaccio/kingspan-connect-sensor) -->\n\nAPI to get oil tank from [Kingspan SENSiT sensors](https://www.kingspan.com/gb/en-gb/products/tank-monitoring-systems/remote-tank-monitoring/sensit-smart-wifi-tank-level-monitoring-kit)\n\nTo make use of the API, you will need the credentials you used to register with the App. You do not need other details such as the tank ID as these are already associated with your account.\n\n## Installation\n\n``` bash\npython3 -m pip kingspan-connect-sensor\n```\n\n## Usage\n\n*NOTE* from version 2.0.0, the API changes to use attributes rather than methods for tank parameters.\n\nReading documents:\n\n``` python\nfrom connectsensor import SensorClient\n\nclient = SensorClient()\nclient.login("test@example.com", "s3cret")\ntanks = client.tanks\ntank_level = tanks[0].level\n```\n\nThe `tanks` method returns a `Tanks` object which can be queried for the status of the specific tank.\n\n## Async Usage\n\nFrom version 2.0.0, an asyncio verison of the client is available:\n\n``` python\nasync with AsyncSensorClient() as client:\n    await client.login("test@example.com", "s3cret")\n    tanks = await client.tanks\n    tank_level = await tanks[0].level\n    tank_capcity = await tanks[0].capacity\n    tank_percent = 100 * (tank_level / tank_percent)\n    print(f"Tank is {tank_percent:.1f}% full")\n```\n\n## Tanks object\n\n`history` returns a Pandas dataframe with all entries available from the web API, sorted by logging time. There should be one record per day. The dataframe has the following entries:\n\n* `reading_date`: a datetime object indicating the time a measurement was made\n* `level_percent`: integer percentage full\n* `level_litres`: number of lites in the tank\n\n## Scripts\n\nReporting on the current status of the tank using `kingspan-status`:\n\n``` bash\n% kingspan-status --username=test@example.com --password=s3cret\n\nHome Tank:\n Capacity = 2000\n Serial Number = 20001999\n Model = Unknown\n Level = 90% (1148 litres)\n Last Read = 2021-10-09 00:42:47.947000\n\nHistory:\n Reading date           %Full  Litres\n 30-Jan-2021 00:29      94     1224 \n 31-Jan-2021 00:59      80     1040 \n 01-Feb-2021 00:29      78     1008 \n 02-Feb-2021 00:59      76     986  \n```\n\n`kingspan-notifier` can be used to check the status of a tabk and report via email when the tank is likely to run out of oil.\n\n``` bash\n% kingspan-notifier --config kingspan.ini\nCurrent level 1148 litres\nNo notification; 196 days oil remain\n```\n\nCommand line options include:\n\n* `--config CONFIG`: a config file in ini-format\n* `--no-update`: don\'t update cache with new data (defaults to updating the DB cache)\n* `--window WINDOW`: the number of days history to consider (default: 14 days)\n* `--notice NOTICE`: the number of days before out-of-oil forecast to warn (default: 14)\n\nAn example config file is:\n\n``` ini\n[sensit]\nusername=test@example.com\npassword=s3cret\ncache=/home/me/kingspan.db\nstart-date=2021-01-31\n\n[smtp]\nserver=smtp.isp.co.uk\nusername=ispuser\nemail=test@example.com\npassword=smtps3cret\n```\n\nThe cache is an SQLite database and will be intialised if not present.\n',
    'author': 'Jon Connell',
    'author_email': 'python@figsandfudge.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/masaccio/kingspan-connect-sensor',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import argparse
import configparser
import os
import pandas as pd
import smtplib
import sqlite3
import ssl
import sys

cwd = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, cwd + "/../")

from connectsensor import SensorClient, APIError
from datetime import datetime, timedelta


def time_delta_days(td):
    return td.days + (td.seconds / 3600) / 24


def send_email(server, username, password, email, subject, message):
    port = 465
    message = (
        f"From: SENSiT Notifier <{email}>\n"
        f"To: <{email}>\n"
        f"Subject: {subject}\n\n" + message
    )
    context = ssl.create_default_context()
    with smtplib.SMTP_SSL(server, port, context=context) as server:
        server.login(username, password)
        server.sendmail(email, email, message)


def read_config(config_filename):
    try:
        config = configparser.ConfigParser(interpolation=None)
        config.read(config_filename)
    except configparser.Error as e:
        print(config_filename + ": " + str(e))
        sys.exit(1)
    finally:
        return config


def config_value(config, section, key):
    try:
        value = config.get(section, key)
    except configparser.Error as e:
        print(f"Config value '{key}' not found in section '{section}'", file=sys.stderr)
        sys.exit(1)
    finally:
        return value


def read_tank_history(config):
    try:
        client = SensorClient()
        client.login(
            config_value(config, "sensit", "username"),
            config_value(config, "sensit", "password"),
        )
    except APIError as e:
        if "Authentication Failed" in str(e):
            print(
                "Authentication Failed: invalid username or password", file=sys.stderr
            )
        else:  # pragma: no cover
            print("Unknown API error:", e.value, file=sys.stderr)
        sys.exit(1)

    tanks = client.tanks
    return tanks[0].history


def update_tank_cache(config, history, update=False):
    cache_db = config.get("sensit", "cache", fallback=None)
    cache_db = os.path.expanduser(cache_db)
    if cache_db is None:
        return

    start_date = config.get("sensit", "start-date", fallback=None)
    if start_date is not None:
        history = history[history.reading_date >= start_date]

    try:
        conn = sqlite3.connect(cache_db)
        cur = conn.cursor()
    except configparser.BasicInterpolationError as e:
        print("DB error:", str(e))
        sys.exit(1)

    cur.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='history'")
    rows = cur.fetchall()
    if len(rows) > 0:
        old_history = pd.read_sql_query(
            "select * from history;", conn, parse_dates=["reading_date"]
        )
        history = pd.concat([history, old_history]).drop_duplicates()

    if update:
        history.to_sql("history", conn, if_exists="replace", index=False)

    cur.close()
    conn.close()

    return history


def usage_rate(history, threshold):
    if len(history) == 0:
        return 0
    current_level = history.level_litres.iloc[0]
    delta_levels = []
    for index, row in history.iloc[1:].iterrows():
        # Ignore refill days where oil goes up by 'threshold'
        if (row.level_litres / current_level) < threshold:
            delta_levels.append(current_level - row.level_litres)
        current_level = row.level_litres
    return sum(delta_levels) / len(delta_levels)


def forecast_empty(config, history, window):
    time_delta = datetime.today() - timedelta(days=window)
    history = history[history.reading_date >= time_delta]

    threshold = config.get("sensit", "refill-threshold", fallback=1.25)
    rate = usage_rate(history, threshold)
    if rate == 0:
        return 9999.0
    else:
        current_level = int(history.level_litres.tail(1))
        return int(current_level / abs(rate))


parser = argparse.ArgumentParser()
parser.add_argument(
    "-c",
    "--config",
    required=True,
    help="Config file in ini-format",
)
parser.add_argument(
    "-N",
    "--no-update",
    action="store_true",
    default=False,
    help="Don't update cache with new data (deault: update on)",
)
parser.add_argument(
    "-w",
    "--window",
    default=14,
    type=int,
    help="Number of days history to consider (default: 14)",
)
parser.add_argument(
    "-n",
    "--notice",
    default=14,
    type=int,
    help="Number of days before out-of-oil forecast to warn (default: 14)",
)


def main():
    args = parser.parse_args()
    config = read_config(args.config)

    tank_history = read_tank_history(config)
    tank_history = update_tank_cache(config, tank_history, args.no_update)
    days_to_empty = forecast_empty(config, tank_history, args.window)

    print("Current level", tank_history.level_litres.iloc[-1], "litres")
    if days_to_empty < args.notice:
        level_percent = tank_history.level_percent.iloc[-1]
        level_litres = tank_history.level_litres.iloc[-1]

        message = "SENSiT is reporting:\n"
        message += f"    * level at {level_percent}%\n"
        message += f"    * level at {level_litres} litres\n\n"
        message += f"Forecasting empty in {days_to_empty} days\n"

        send_email(
            config_value(config, "smtp", "server"),
            config_value(config, "smtp", "username"),
            config_value(config, "smtp", "password"),
            config_value(config, "smtp", "email"),
            "Low oil warning from SENSiT",
            message,
        )

        print(f"Sent notification: empty in {days_to_empty} days")
    else:
        print(f"No notification; {days_to_empty} days oil remain")


if __name__ == "__main__":
    # execute only if run as a script
    main()



class DBError(Exception):
    """Exception raised for tank history database errors"""

    pass


class APIError(Exception):
    """Exception raised for internal errors from Kingspan API"""

    pass



from async_property import async_property
from datetime import datetime
from urllib.parse import urljoin
from zeep import Client as SoapClient
from zeep import AsyncClient as AsyncSoapClient

from .tank import Tank, AsyncTank
from .exceptions import APIError

DEAFULT_SERVER = "https://www.connectsensor.com/"
WSDL_PATH = "soap/MobileApp.asmx?WSDL"
WSDL_URL = urljoin(DEAFULT_SERVER, WSDL_PATH)


class SensorClient:
    def __init__(self):
        self._soap_client = SoapClient(WSDL_URL)
        self._soap_client.set_ns_prefix(None, "http://mobileapp/")

    def login(self, username, password):
        self._username = username
        self._password = password

        response = self._soap_client.service.SoapMobileAPPAuthenicate_v3(
            emailaddress=username, password=password
        )

        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])

        self._user_id = response["APIUserID"]
        self._tanks = []
        for tank_info in response["Tanks"]["APITankInfo_V3"]:
            self._tanks.append(Tank(self, tank_info["SignalmanNo"]))

    @property
    def tanks(self):
        return self._tanks

    def _get_latest_level(self, signalman_no):
        response = self._soap_client.service.SoapMobileAPPGetLatestLevel_v3(
            userid=self._user_id,
            password=self._password,
            signalmanno=signalman_no,
            culture="en",
        )

        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])  # pragma: no cover
        return response

    def _get_history(self, signalman_no, start_date=None, end_date=None):
        if start_date is None:
            start_date_dt = datetime.fromtimestamp(0)
        if end_date is None:
            end_date_dt = datetime.now()

        response = self._soap_client.service.SoapMobileAPPGetCallHistory_v1(
            userid=self._user_id,
            password=self._password,
            signalmanno=signalman_no,
            startdate=start_date_dt.isoformat(),
            enddate=end_date_dt.isoformat(),
        )

        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])  # pragma: no cover
        return response["Levels"]["APILevel"]


class AsyncSensorClient:
    def __init__(self, base=DEAFULT_SERVER):
        url = urljoin(base, WSDL_PATH)
        self._soap_client = AsyncSoapClient(url)
        self._soap_client.set_ns_prefix(None, "http://mobileapp/")

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_t, exc_v, exc_tb):
        pass

    async def login(self, username, password):
        self._username = username
        self._password = password

        response = await self._soap_client.service.SoapMobileAPPAuthenicate_v3(
            emailaddress=username, password=password
        )

        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])

        self._user_id = response["APIUserID"]
        self._tanks = []
        for tank_info in response["Tanks"]["APITankInfo_V3"]:
            self._tanks.append(AsyncTank(self, tank_info["SignalmanNo"]))
        return response

    @async_property
    async def tanks(self):
        return self._tanks

    async def _get_latest_level(self, signalman_no):
        response = await self._soap_client.service.SoapMobileAPPGetLatestLevel_v3(
            userid=self._user_id,
            password=self._password,
            signalmanno=signalman_no,
            culture="en",
        )
        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])  # pragma: no cover
        return response

    async def _get_history(self, signalman_no, start_date=None, end_date=None):
        if start_date is None:
            start_date_dt = datetime.fromtimestamp(0)
        if end_date is None:
            end_date_dt = datetime.now()

        response = await self._soap_client.service.SoapMobileAPPGetCallHistory_v1(
            userid=self._user_id,
            password=self._password,
            signalmanno=signalman_no,
            startdate=start_date_dt.isoformat(),
            enddate=end_date_dt.isoformat(),
        )

        if response["APIResult"]["Code"] != 0:
            raise APIError(response["APIResult"]["Description"])  # pragma: no cover
        return response["Levels"]["APILevel"]
